@@ -6,14 +6,31 @@
 
 #include "version.h"
 #include "sysfs.h"
+#include "jedec.h"
 #include "osutils.h"
 
+#include <stdlib.h>
 #include <regex.h>
 
-__ID("@(#) $Id: heuristics.cc 2069 2009-02-12 22:53:09Z lyonel $");
+__ID("@(#) $Id$");
 
 string guessBusInfo(const string & info)
 {
+                                                  // NVMe: "nvme"host"n"namespace
+  if(matches(info, "^nvme[0-9]+n[0-9]+$"))
+  {
+    string s = info;
+    size_t n = s.rfind("n");
+    s[n] = ':';
+    return "nvme@" + s.substr(4, string::npos);
+  }
+
+                                                  // SD/MMC: "mmc"host:address
+  if(matches(info, "^mmc[0-9]+(:[[:xdigit:]]+)+$"))
+  {
+    return "mmc@" + info.substr(3, string::npos);
+  }
+
                                                   // 2.6-style PCI
   if(matches(info,"^[[:xdigit:]][[:xdigit:]][[:xdigit:]][[:xdigit:]]:[[:xdigit:]][[:xdigit:]]:[[:xdigit:]][[:xdigit:]]\\.[[:xdigit:]]$"))
   {
@@ -90,7 +107,8 @@ hwNode * guessParent(const hwNode & child, hwNode & base)
 
 static const char *disk_manufacturers[] =
 {
-  "^ST.+", "Seagate",
+  "^ST(?!I\\ ).+", "Seagate",
+  "^STI\\ .+", "SimpleTech",
   "^D...-.+", "IBM",
   "^IBM.+", "IBM",
   "^HITACHI.+", "Hitachi",
@@ -107,6 +125,7 @@ static const char *disk_manufacturers[] =
   "FIREBALL.+", "Quantum",
   "^WDC.+", "Western Digital",
   "WD.+", "Western Digital",
+  "^VBOX.+", "VirtualBox",
   NULL, NULL
 };
 
@@ -115,6 +134,9 @@ bool guessVendor(hwNode & device)
   int i = 0;
   bool result = false;
 
+
+  device.setVendor(jedec_resolve(device.getVendor()));
+
   if(device.getVendor() != "")
     return false;
 
@@ -133,3 +155,41 @@ bool guessVendor(hwNode & device)
   return result;
 }
 
+
+static string parsehex(const string & s)
+{
+  unsigned int i = 0;
+  string result = "";
+
+  if(matches(s,"^0x[[:xdigit:]][[:xdigit:]]+$"))
+  {
+    for(i=2; i<s.length(); i+=2)
+    {
+      string c = s.substr(i,2);
+      static char code[2];
+
+      code[0] = strtol(c.c_str(), NULL, 16);
+      code[1] = '\0';
+      
+      if(code[0] < ' ') return s;
+      result += string(code);
+    }
+   }
+   else
+     return s;
+
+  return result;
+}
+
+bool guessProduct(hwNode & device)
+{
+  string product = device.getProduct();
+
+  if(product == "")
+    return false;
+
+  device.setProduct(parsehex(product));
+
+  return true;
+}
+ 
