@@ -6,22 +6,27 @@
 #include "osutils.h"
 #include "options.h"
 
+#include <cstring>
+#include <cstdlib>
 #include <iostream>
 #include <fstream>
 #include <sys/utsname.h>
+#include <stdlib.h>
+#include <string.h>
 #include <libgen.h>
 
-static char *id = "@(#) $Id: engine.cc 1818 2007-08-05 21:11:59Z lyonel $";
+static const char *id = "@(#) $Id$";
 
 extern "C"
 {
 #include "support.h"
 };
 
-#define AUTOMATIC "automatic"
-#define LSHW_XML "lshw XML format"
-#define PLAIN_TEXT "plain text document"
-#define HTML "HTML document"
+#define AUTOMATIC "automatic file format"
+#define LSHW_XML "lshw XML format (.lshw, .xml)"
+#define PLAIN_TEXT "plain text document (.text, .txt)"
+#define JSON "JavaScript Object Notation document (.json)"
+#define HTML "HTML document (.html, .htm)"
 
 #define YIELD()  while(gtk_events_pending()) gtk_main_iteration()
 
@@ -30,9 +35,14 @@ static hwNode *displayed = NULL;
 static hwNode *selected1 = NULL;
 static hwNode *selected2 = NULL;
 static hwNode *selected3 = NULL;
-static GtkWidget *statusbar = NULL;
 
 extern GtkWidget *mainwindow;
+extern GtkWidget *list1, *list2, *list3;
+extern GtkWidget *description;
+extern GtkWidget *statusbar;
+extern GHashTable *pixbufs;
+extern GSimpleAction *go_up_action;
+extern GSimpleAction *save_action;
 
 enum
 {
@@ -206,7 +216,6 @@ static void create_tags (GtkTextBuffer *buffer)
 
 static void display(GtkWidget * mainwindow)
 {
-  GtkWidget * description = lookup_widget(mainwindow, "description");
   GtkTextBuffer *buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (description));
 
   gtk_text_buffer_set_text(buffer, "", -1);
@@ -218,7 +227,7 @@ static void display(GtkWidget * mainwindow)
     create_tags(buffer);
 
     string hwpath = gethwpath(*displayed, container);
-    printmarkup(*displayed, GTK_TEXT_VIEW(description), hwpath);
+    printmarkup(*displayed, GTK_TEXT_VIEW(description), hwpath, pixbufs);
   }
 }
 
@@ -244,22 +253,11 @@ void refresh(GtkWidget *mainwindow)
 {
   hwNode computer("computer", hw::system);
   static bool lock = false;
-  GtkWidget * description = lookup_widget(mainwindow, "description");
-  GtkWidget * go_up_button = lookup_widget(mainwindow, "upbutton");
-  GtkWidget * save_button = lookup_widget(mainwindow, "savebutton");
-  GtkWidget * menu = lookup_widget(mainwindow, "menu");
-  GtkWidget * save_menuitem = lookup_widget(menu, "save");
 
   if(lock) return;
 
   lock = true;
-  gtk_widget_set_sensitive(save_button, FALSE);
-  gtk_widget_set_sensitive(save_menuitem, FALSE);
-
-  GtkWidget * list1 = lookup_widget(mainwindow, "treeview1");
-  GtkWidget * list2 = lookup_widget(mainwindow, "treeview2");
-  GtkWidget * list3 = lookup_widget(mainwindow, "treeview3");
-  statusbar = lookup_widget(mainwindow, "statusbar");
+  g_simple_action_set_enabled(save_action, FALSE);
 
   populate_sublist(list1, NULL);
   populate_sublist(list2, NULL);
@@ -274,9 +272,8 @@ void refresh(GtkWidget *mainwindow)
   status(NULL);
   displayed = container.addChild(computer);
 
-  gtk_widget_set_sensitive(go_up_button, FALSE);
-  gtk_widget_set_sensitive(save_button, TRUE);
-  gtk_widget_set_sensitive(save_menuitem, TRUE);
+  g_simple_action_set_enabled(go_up_action, FALSE);
+  g_simple_action_set_enabled(save_action, TRUE);
 
   selected1 = NULL;
   selected2 = NULL;
@@ -296,8 +293,6 @@ void change_selection(unsigned list, GtkTreeView *treeview)
   GtkTreeSelection *selection;
   GtkTreeModel *model;
   GtkTreeIter   iter;
-  GtkWidget * list2 = lookup_widget(mainwindow, "treeview2");
-  GtkWidget * list3 = lookup_widget(mainwindow, "treeview3");
 
   model = gtk_tree_view_get_model(treeview);
 
@@ -338,10 +333,6 @@ void browse(unsigned list, GtkTreeView *treeview)
   GtkTreeModel     *model;
   GtkTreeIter       iter;
   hwNode *n = NULL;
-  GtkWidget * list1 = lookup_widget(mainwindow, "treeview1");
-  GtkWidget * list2 = lookup_widget(mainwindow, "treeview2");
-  GtkWidget * list3 = lookup_widget(mainwindow, "treeview3");
-  GtkWidget * go_up_button = lookup_widget(mainwindow, "upbutton");
 
   selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
   if (gtk_tree_selection_get_selected(selection, &model, &iter))
@@ -386,10 +377,7 @@ void browse(unsigned list, GtkTreeView *treeview)
       break;
   }
 
-  if(selected1 && (find_parent(selected1, &container)!= &container))
-    gtk_widget_set_sensitive(go_up_button, 1);
-  else
-    gtk_widget_set_sensitive(go_up_button, 0);
+  g_simple_action_set_enabled(go_up_action, selected1 && (find_parent(selected1, &container)!= &container));
 
   (void) &::id;                                   // avoid warning "id defined but not used"
 }
@@ -397,11 +385,6 @@ void browse(unsigned list, GtkTreeView *treeview)
 
 void go_back(GtkWidget *mainwindow)
 {
-  GtkWidget * list1 = lookup_widget(mainwindow, "treeview1");
-  GtkWidget * list2 = lookup_widget(mainwindow, "treeview2");
-  GtkWidget * list3 = lookup_widget(mainwindow, "treeview3");
-  GtkWidget * go_up_button = lookup_widget(mainwindow, "upbutton");
-
   if(selected1 && (find_parent(selected1, &container)!= &container))
   {
     selected3 = selected2;
@@ -416,10 +399,7 @@ void go_back(GtkWidget *mainwindow)
       displayed = find_parent(displayed, &container);
   }
 
-  if(selected1 && (find_parent(selected1, &container)!= &container))
-    gtk_widget_set_sensitive(go_up_button, 1);
-  else
-    gtk_widget_set_sensitive(go_up_button, 0);
+  g_simple_action_set_enabled(go_up_action, selected1 && (find_parent(selected1, &container)!= &container));
 
   display(mainwindow);
 }
@@ -437,10 +417,13 @@ static const char *guess_format(char *s)
   if(!strcasecmp(".text", dot) || !strcasecmp(".txt", dot))
     return PLAIN_TEXT;
 
+  if(!strcasecmp(".json", dot))
+    return JSON;
+
   return LSHW_XML;
 }
 
-static char *fix_filename(char *s, char *extension)
+static char *fix_filename(char *s, const char *extension)
 {
   char *dot = strrchr(s, '.');
 
@@ -470,7 +453,7 @@ static void redirect_cout(std::ofstream &out, bool enable = true)
 void save_as(GtkWidget *mainwindow)
 {
   struct utsname buf;
-  GtkWidget *dialog = NULL;
+  GtkFileChooserNative *dialog = NULL;
   GtkWidget *sanitize = NULL;
   GtkFileFilter *filter = NULL;
   bool proceed = true;
@@ -479,12 +462,11 @@ void save_as(GtkWidget *mainwindow)
   if(!computer)		// nothing to save
     return;
 
-  dialog = gtk_file_chooser_dialog_new ("Save hardware configuration",
+  dialog = gtk_file_chooser_native_new ("Save hardware configuration",
 				      GTK_WINDOW(mainwindow),
 				      GTK_FILE_CHOOSER_ACTION_SAVE,
-				      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-				      GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
-				      NULL);
+				      "_Save",
+				      "_Cancel");
   //gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (dialog), TRUE);
   sanitize = gtk_check_button_new_with_label("Anonymize output");
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(sanitize), enabled("output:sanitize")?TRUE:FALSE);
@@ -512,11 +494,16 @@ void save_as(GtkWidget *mainwindow)
   gtk_file_filter_add_pattern(filter, "*.txt");
   gtk_file_filter_add_mime_type(filter, "text/plain");
   gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(dialog), filter);
+  filter = gtk_file_filter_new ();
+  gtk_file_filter_set_name(filter, JSON);
+  gtk_file_filter_add_pattern(filter, "*.json");
+  gtk_file_filter_add_mime_type(filter, "application/json");
+  gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(dialog), filter);
 
   if(uname(&buf)==0)
     gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (dialog), buf.nodename);
 
-  if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
+  if (gtk_native_dialog_run (GTK_NATIVE_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
   {
     char *filename;
 
@@ -544,6 +531,9 @@ void save_as(GtkWidget *mainwindow)
         else
         if(strcmp(filtername, PLAIN_TEXT)==0)
           filename = fix_filename(filename, "txt");
+        else
+        if(strcmp(filtername, JSON)==0)
+          filename = fix_filename(filename, "json");
       }
 
       if(exists(filename))		// existing file
@@ -551,15 +541,15 @@ void save_as(GtkWidget *mainwindow)
         char * buffer1 = g_strdup(filename);
         char * buffer2 = g_strdup(filename);
 
-        GtkWidget *dialog = gtk_message_dialog_new (GTK_WINDOW(mainwindow),
+        GtkWidget *dialog = gtk_message_dialog_new_with_markup (GTK_WINDOW(mainwindow),
                                   GTK_DIALOG_DESTROY_WITH_PARENT,
                                   GTK_MESSAGE_WARNING,
                                   GTK_BUTTONS_NONE,
-                                  "A file named '%s' already exists in folder '%s'.\n\nDo you want to overwrite it?",
+                                  "A file named <i><tt>%s</tt></i> already exists in folder <tt>%s</tt>.\n\nDo you want to overwrite it?",
                                   basename(buffer1), dirname(buffer2));
         gtk_dialog_add_buttons(GTK_DIALOG(dialog), 
-				  GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-				  "Overwrite", GTK_RESPONSE_ACCEPT,
+				  "_Cancel", GTK_RESPONSE_CANCEL,
+				  "_Overwrite", GTK_RESPONSE_ACCEPT,
                                   NULL);
         proceed = (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT);
         gtk_widget_destroy (dialog);
@@ -592,10 +582,18 @@ void save_as(GtkWidget *mainwindow)
           print(*computer, false);
           redirect_cout(out, false);
         }
+	else
+        if(strcmp(filtername, JSON)==0)
+        {
+          std::ofstream out(filename);
+          redirect_cout(out);
+          cout << computer->asJSON() << endl;
+          redirect_cout(out, false);
+        }
       }
       g_free (filename);
     }
   }
 
-  gtk_widget_destroy (dialog);
+  g_object_unref (dialog);
 }
